@@ -28,7 +28,7 @@
         return(FALSE)
     }
     if (inherits(x, "haven_labelled")) {
-        return(!any(is.na(suppressWarnings(as.numeric(names(attr(x, "labels")))))))
+        return(Recall(unclass(x)) && !any(is.na(suppressWarnings(as.numeric(names(attr(x, "labels")))))))
     }
     if (is.numeric(x)) {
         return(TRUE)
@@ -36,5 +36,8 @@
     if (is.factor(x)) {
         return(!any(is.na(suppressWarnings(as.numeric(levels(x))))))
     }
+    if (any(grepl("[^!-~]", x))) {
+        return(FALSE)
+    }
     return(!any(is.na(suppressWarnings(as.numeric(na.omit(x))))))
 }
